/******************************************************************************************
 
 This program solves reacrtion-diffusion equations using the explicit method.
 
 Reacrtion-diffusion equations are ut = du uxx + f(u,v) and vt = dv vxx + g(u,v).
 
 Time loop in this program consists of
 "Calculation part" and "Draw Part".
 
 Calculation part is controlled by the interval parameter INTV.
 This numerical result is visualized by using "g_color_map" made in thie program.
 
*******************************************************************************************/

#include<stdio.h>
#include<stdlib.h>
#include<glsc3d_3.h>

#define N (100)
#define M (100)

//////////// Color map: g_color_map ////////////
void g_color_map(double Array[][M+2],
                 double dx_width, double dy_width,
                 double L_bottom_x, double L_bottom_y,
                 double max, double min
)
{
    int Fine_Grid = 1;
    for (int j=0; j < M+2; j++) {
        for (int i=0; i < N+2; i++) {
            if (Fine_Grid == 0) {
                g_area_color((Array[i][j]-min)/(max-min), 0, 1.0-(Array[i][j]-min)/(max-min), 0.5);
                g_box_center_2D(dx_width * (i) + L_bottom_x, dy_width * (j) + L_bottom_y, dx_width, dy_width, G_NO, G_YES);
            }
            
            if (Fine_Grid == 1) {
                double tmpColor;
                if (j-1<0) tmpColor = Array[i][j];
                else tmpColor = Array[i][j]*0.5 + Array[i][j-1]*0.5;
                g_area_color((tmpColor-min)/(max-min), 0, 1.0-(tmpColor-min)/(max-min), 1);
                g_triangle_2D(dx_width * (i) + L_bottom_x, dy_width * (j) + L_bottom_y,
                              dx_width * (i) + L_bottom_x - dx_width*0.5, dy_width * (j) + L_bottom_y - dy_width*0.5,
                              dx_width * (i) + L_bottom_x + dx_width*0.5, dy_width * (j) + L_bottom_y - dy_width*0.5,
                              G_NO, G_YES);
                
                if (i+1>=N+2) tmpColor = Array[i][j];
                else tmpColor = Array[i][j]*0.5 + Array[i+1][j]*0.5;
                g_area_color((tmpColor-min)/(max-min), 0, 1.0-(tmpColor-min)/(max-min), 1);
                g_triangle_2D(dx_width * (i) + L_bottom_x, dy_width * (j) + L_bottom_y,
                              dx_width * (i) + L_bottom_x + dx_width*0.5, dy_width * (j) + L_bottom_y - dy_width*0.5,
                              dx_width * (i) + L_bottom_x + dx_width*0.5, dy_width * (j) + L_bottom_y + dy_width*0.5,
                              G_NO, G_YES);
                
                if (j+1>=M+2) tmpColor = Array[i][j];
                else tmpColor = Array[i][j]*0.5 + Array[i][j+1]*0.5;
                g_area_color((tmpColor-min)/(max-min), 0, 1.0-(tmpColor-min)/(max-min), 1);
                g_triangle_2D(dx_width * (i) + L_bottom_x, dy_width * (j) + L_bottom_y,
                              dx_width * (i) + L_bottom_x + dx_width*0.5, dy_width * (j) + L_bottom_y + dy_width*0.5,
                              dx_width * (i) + L_bottom_x - dx_width*0.5, dy_width * (j) + L_bottom_y + dy_width*0.5,
                              G_NO, G_YES);
                
                if (i-1<0) tmpColor = Array[i][j];
                else tmpColor = Array[i][j]*0.5 + Array[i-1][j]*0.5;
                g_area_color((tmpColor-min)/(max-min), 0, 1.0-(tmpColor-min)/(max-min), 1);
                g_triangle_2D(dx_width * (i) + L_bottom_x, dy_width * (j) + L_bottom_y,
                              dx_width * (i) + L_bottom_x - dx_width*0.5, dy_width * (j) + L_bottom_y + dy_width*0.5,
                              dx_width * (i) + L_bottom_x - dx_width*0.5, dy_width * (j) + L_bottom_y - dy_width*0.5,
                              G_NO, G_YES);
            }
        }
    }
}

double f(double u, double v)
{
    return u * (1 - u * u) - v;
}
double g(double u, double v)
{
    return 3.0 * u - 2.0 * v;
//    return 3.0 * (u + 0.05) - 2.0 * v;
//    return 3.0 * (u - 0.05) - 2.0 * v;
}

int main()
{
    double  Lx = 100.0;
    double  Ly = 100.0;
    double  dx = Lx / N;
    double  dy = Ly / M;
    double  dt = 0.001;
    double  du = 1.0;
    double  dv = 10.0;
    
    double  lambda_u = du * dt / (dx * dx);
    double  lambda_v = dv * dt / (dy * dy);
    
    double  u0[N + 2][M + 2];
    double  u1[N + 2][M + 2];
    double  v0[N + 2][M + 2];
    double  v1[N + 2][M + 2];

    int INTV = 200;
    
	g_init("Window", 600, 600);	//Pixel Size
    
    g_def_scale_2D(0,                   //ID
                   -Lx*0.5, Lx*0.5,		//xmin,xmax
                   -Ly*0.5, Ly*0.5,     //ymin,ymax
                   20.0, 20.0,          //Window (Left, Top) Position
                   560, 560);           //Window Size (x,y)
    //Set initial calculation
    {
        //Step1
        double u_Init_Sum = 0.0;
        double v_Init_Sum = 0.0;
        
        unsigned int seed = 2;   //Seed for random number generated by rand()
        srand(seed);
        
        for(int j = 1;j < M + 1; j ++)
        {
            for(int i = 1;i < N + 1; i ++)
            {
                double Max = 1.0; //Maximum of random number
                double Min = 0.0; //Minimum of random number
                double RandNumber = Min + (int)(rand()*(Max-Min+1.0)/(1.0+RAND_MAX));
                u0[i][j] = 2.0 * (RandNumber - 0.5);
                v0[i][j] = 2.0 * (RandNumber - 0.5);
            
                u_Init_Sum += u0[i][j] * dx * dy;
                v_Init_Sum += v0[i][j] * dx * dy;
            }
        }
        
        //Step2
        for(int j = 1;j < M + 1; j ++)
        {
            u0[0][j] = u0[N][j];
            u0[N + 1][j] = u0[1][j];
            v0[0][j] = v0[N][j];
            v0[N + 1][j] = v0[1][j];
        }
        for(int i = 0;i < N + 2; i ++)
        {
            u0[i][0] = u0[i][M];
            u0[i][M + 1] = u0[i][1];
            v0[i][0] = v0[i][M];
            v0[i][M + 1] = v0[i][1];
        }
    }
    
    //////////// Start time loop ////////////
    for (int i_time = 0; ; i_time++) {

        //////////// Draw Part ////////////
        if (i_time%INTV == 0) {
            g_cls();					//Clear window
            g_sel_scale(0);				//Select Virtual scale
            g_line_color(0.0, 0.0, 0.0, 1.0);
            g_boundary();				//Draw Boundary
            
            g_color_map(u0, dx, dy, -Lx*0.5 + dx * 0.5, -Ly*0.5 + dy * 0.5, 0.5, -0.5);
            
            g_finish();					//flush Draw buffer
            g_sleep(0.01);				//Sleep 0.01 sec
        }
        
        //////////// Calculation Part ////////////
        {
        //Step3
            for(int j = 1;j < M + 1; j ++)
            {
                for(int i = 1;i < N + 1; i ++)
                {
                    u1[i][j] = u0[i][j]
                    + lambda_u * (u0[i - 1][j] - 2 * u0[i][j] + u0[i + 1][j])
                    + lambda_u * (u0[i][j - 1] - 2 * u0[i][j] + u0[i][j + 1])
                    + dt * f(u0[i][j], v0[i][j]);
                    
                    v1[i][j] = v0[i][j]
                    + lambda_v * (v0[i - 1][j] - 2 * v0[i][j] + v0[i + 1][j])
                    + lambda_v * (v0[i][j - 1] - 2 * v0[i][j] + v0[i][j + 1])
                    + dt * g(u0[i][j], v0[i][j]);
                }
            }
            //Step4
            for(int j = 1;j < M + 1; j ++)
            {
                for(int i = 1;i < N + 1; i ++)
                {
                    u0[i][j] = u1[i][j];
                    v0[i][j] = v1[i][j];
                }
            }
            //Step5
            for(int j = 1;j < M + 1; j ++)
            {
                u0[0][j] = u0[N][j];
                u0[N + 1][j] = u0[1][j];
                v0[0][j] = v0[N][j];
                v0[N + 1][j] = v0[1][j];
            }
            for(int i = 0;i < N + 2; i ++)
            {
                u0[i][0] = u0[i][M];
                u0[i][M + 1] = u0[i][1];
                v0[i][0] = v0[i][M];
                v0[i][M + 1] = v0[i][1];
            }
        }
    
        //////////// Check part of maximum and minimum of u and v in order to use g_color_map ////////////
//        double u_Max = u0[0][0];
//        double u_Min = u0[0][0];
//        double v_Max = v0[0][0];
//        double v_Min = v0[0][0];
//        for(int j = 1;j < M + 1; j ++)
//        {
//            for(int i = 1;i < N + 1; i ++)
//            {
//                if (u_Max < u0[i][j]) u_Max = u0[i][j];
//                if (u_Min > u0[i][j]) u_Min = u0[i][j];
//                if (v_Max < v0[i][j]) v_Max = v0[i][j];
//                if (v_Min > v0[i][j]) v_Min = v0[i][j];
//            }
//        }
//        printf("%d th: u_Max = % 2.15f\n", i_time, u_Max);
//        printf("%d th: u_Min = % 2.15f\n", i_time, u_Min);
//        printf("%d th: v_Max = % 2.15f\n", i_time, v_Max);
//        printf("%d th: v_Min = % 2.15f\n", i_time, v_Min);
    }
	return 0;
}
